/* This file was generated by syncqt. */
#ifndef QT_QTQUICK_VERSION_H
#define QT_QTQUICK_VERSION_H

#define QTQUICK_VERSION_STR "5.15.16"

#define QTQUICK_VERSION 0x050F10

#endif // QT_QTQUICK_VERSION_H
